import requests 
import logging
import json
import traceback

import sys
sys.path.append('.')
from appsUtil import AppsUtil

util = AppsUtil()


class AppsRequestLib():

	certs = None
	cert = None
	url = None
	headers = None
	services = None
	path= None
	correlationSupportedServices = None
	env = None
	
	def __init__(self):
		global certs
		global cert
		global url
		global headers
		global services
		global path 
		global correlationSupportedServices
		global env 
		data = None
		with open('../../config/master_config.json') as data_file:    
			data = json.load(data_file)
		if data!=None:
			env = data['env']
			certs = data['certs'][0]
			path = '../../common/certs/'
			ca_file=certs['cacert']
			pem_file = certs['pem']
			dkey_file = certs['dkey']
			
			services = data['services'][0]
			correlationSupportedServices = data['correlationSupportedServices']
			logging.info(" got this correlationSupportedServices ",correlationSupportedServices)
			
		
		certs=path+'/'+ca_file
		cert=(path+'/'+pem_file,path+'/'+dkey_file)
	
	def do_get(self,newUrl=None,newHeaders=None,newParams=None,data=None,newCerts=None,ignore_certs=False):
		global certs
		global cert
		global path
		defaultCerts = certs
		r = None
		try:
			if newCerts!=None:			
				certs = path+'/'+newCerts
			logging.debug ("certs -> ",certs)
			logging.debug ("params ",newParams)
			if ignore_certs==False:
				r = requests.get(newUrl, headers=newHeaders,params=newParams,verify=certs,cert=cert,data=data)
			else:
				r = requests.get(newUrl, headers=newHeaders,params=newParams,data=data)
			logging.debug("REQUEST:   "+str(r.request))
			logging.debug("HEADERS:   "+str(r.headers))
			logging.debug("URL:       "+r.url)			
			logging.debug("STATUS:    "+str(r.status_code))
		except Exception as exc:
			logging.debug("Exception while sending request : ",exc)
			traceback.print_stack()

		certs = defaultCerts
		return r


	def do_delete(self,newUrl=None,newHeaders=None,newParams=None,data=None,newCerts=None,ignore_certs=False):
		global certs
		global cert
		global path
		defaultCerts = certs
		r = None
		try:
			r = None
			if newCerts!=None:
				certs = path+'/'+newCerts
			if ignore_certs==False:
				r = requests.delete(newUrl, headers=newHeaders,params=newParams,verify=certs,cert=cert)
			else:
				r = requests.delete(newUrl, headers=newHeaders,params=newParams)
			if r!=None:
				logging.debug("response status code: ",r.status_code)
		except Exception as exc:
			logging.debug("Exception while sending request : ",exc)
			traceback.print_stack()
			
		certs = defaultCerts
		return r

	def do_post(self,newUrl=None,newHeaders=None,params=None,newData=None,newCerts=None,method=None,ignore_certs=False):
		global certs
		global cert
		global path
		r = None
		defaultCerts= certs 

		try:
			if newCerts!=None:
				certs = path+'/'+newCerts 	
			if method == 'POST':
				if ignore_certs==False:
					r = requests.post(url,headers=newHeaders,data=newData,verify=certs,cert=cert)
				else:
					r = requests.post(url,headers=newHeaders,data=newData)
			elif method == 'PUT':
				if ignore_certs==False:
					r = requests.put(url,headers=newHeaders,data=newData,verify=certs,cert=cert)
				else:
					r = requests.put(url,headers=newHeaders,data=newData)
	
			elif method=='PATCH':
				if ignore_certs==False:
					r = requests.patch(url,headers=newHeaders,data=newData,verify=certs,cert=cert)
				else:
					r = requests.patch(url,headers=newHeaders,data=newData)
			if (r!=None):
				print ("REQUEST:   "+str(r.request))
				print ("HEADERS:   "+str(r.headers))
				print ("URL:       "+r.url)				
		
			print ("STATUS:    "+str(r.status_code))
			print ("STATUS:    "+r.text)
		except Exception as exc:
			print ("Exception while sending request : ",exc)
			traceback.print_stack()
		certs = defaultCerts
		
		

		#print ("JSON RESPONSE: "+str(r.json()))
		
		return r

	
	def do_request(self,svcName,method,path=None,url=None,headers=None):
		if method== 'GET':
			result = self.do_get(path,url,headers)
		return result

	
	def request(self,svcName,method,path=None,params=None,data=None,certs=None,newHeaders=None,ignore_certs=False):
		global url
		global headers
		svc = services[svcName]
		print ("******************************************************************************")
		logging.debug("Processing new request:")
		logging.debug("Service : ",svcName)
		logging.debug("Method : ",method)
		logging.debug("Path : ",path)
		logging.debug("Params : ",params)
		logging.debug("Data :",data)
		logging.debug("Certs : ",certs)
		logging.debug("Headers : ",newHeaders)
		logging.debug("ignore certs : ",ignore_certs)
		url = svc['url']
		if newHeaders!=None:
			headers = newHeaders
		else:
			headers = svc['headers'][0]
		
		corService = False
		if svcName in correlationSupportedServices:
			corService = True
			settingId = False
			randNumber = util.generateRandomNumber()
			print (" random number ",randNumber) 
			if randNumber%20==0:
				settingId = True
				corId = util.generateRandomString(15)
				print ("Setting X-Correlation-Id -------- ",corId)
				headers['X-Correlation-Id'] =  corId
			else:
				print ("Not setting any cor id!!!!!!")
		result = None
		try:
			if path!=None:
				url = url+path
			if  method == 'GET':
				result = self.do_get(newUrl=url,newHeaders=headers,newParams=params,newCerts=certs,data=data,ignore_certs=ignore_certs)
			elif method == 'POST' or method=='PUT' or method =='PATCH':
				
				result = self.do_post(newUrl=url,newHeaders=headers,newData=data,newCerts=certs,method=method,ignore_certs=ignore_certs)
			elif method=='DELETE':
				result = self.do_delete(newUrl=url,newHeaders=headers,newParams=params,newCerts=certs,ignore_certs=ignore_certs)
			print ("Want to see response headers -> ",result.headers)
			
			if corService:
				if settingId:
					print ("got to fix the recursive design issue!")
					#esLib.search(csapi_constants.rpnsIndex, 'txnId='+corId, size='10', time='10m')
		except:
			print ("exception processing request! ")
		newHeaders = None
		logging.debug("response status code :",result.status_code)
		print ("******************************************************************************")
		
		return result
	
	def get_env(self):
		global env
		return env	
			
